/*
 * define file structure constants
 */

#define FBLK_SIZE 512
#define FBLK_BITS 9
#define XTNT_HDR 16
#define BITMAP_BLK 2
#define DISK_DESC_BLK 1
#define MIN_FILE_SIZE 10
#define MIN_INCREMENT 4
#define RANGE_INCREMENTS 0x7c

/*
 * define bits in file status and attribute bytes
 */

#define _FILE_MODIFIED	0x10
#define _USED			0x40
#define _FILE_BUSY		0x80

/*
 * fp_flags bits
 */

#define USE_XHDR_CACHE		0x0001
#define UPDATE_XHDR_CACHE	0x0002
#define SEEKING				0x0004

/*
 * define operating system constants
 */

#define NCPFN 16


struct xtnt_hdr {
	unsigned prev_xtnt;
	unsigned ext_prev_xtnt;
	unsigned next_xtnt;
	unsigned ext_next_xtnt;
	unsigned size_xtnt;
	unsigned ext_size_xtnt;
	unsigned bound_xtnt;
	unsigned ext_bound_xtnt;
	};

struct dir_entry {
	char fstat;
	unsigned ffirst_xtnt;
	unsigned ext_ffirst_xtnt;
	unsigned flast_xtnt;
	unsigned ext_flast_xtnt;
	unsigned fnum_blks;
	unsigned ext_fnum_blks;
	unsigned fnum_xtnt;
	char fowner;
	char fgroup;
	char fspare[6];
	char ftype;
	char fgperms;
	char fperms;
	char fattr;
	unsigned fdate[2];
	char fname[NCPFN + 1];
	};

struct dir_xtnt {
	unsigned parent_xtnt;
	unsigned dir_index;
	struct dir_entry directory[0];
	};

struct disk_desc_blk {
	unsigned parent_xtnt;
	unsigned dir_index;
	unsigned disk_id[2];
	char unused[sizeof(struct dir_entry) - sizeof(disk_id)];
	struct dir_entry disk_root_dir;
	};

struct file_iocb {
	struct iocb _iocb;			/* 00 */
	char _drive;				/* 14 */
	char _mode;					/* 15 */
	char _owner;				/* 16 */
	char _group;				/* 17 */
	char _fspare[2];			/* 18 */
	unsigned _first_xtnt;		/* 1a */
	unsigned _ext_first_xtnt;	/* 1c */
	unsigned _last_xtnt;		/* 1e */
	unsigned _ext_last_xtnt;	/* 20 */
	unsigned _num_blks;			/* 22 */
	unsigned _ext_num_blks;		/* 24 */
	unsigned _num_xtnt;			/* 26 */
	unsigned _cur_xtnt;			/* 28 */
	unsigned _ext_cur_xtnt;		/* 2a */
	unsigned _max_cnt;			/* 2c */
	unsigned _cur_blk_offset;	/* 2e */
	unsigned _parent_xtnt;		/* 30 */
	unsigned _ext_parent_xtnt;	/* 32 */
	unsigned _xhdr_list;		/* 34 */
	unsigned _fp_flags;			/* 36 */
	unsigned _prev_num_xtnts;	/* 38 */
	unsigned _prev_num_blks;	/* 3a */
	int _dir_index;				/* 3c */
	int _temp;					/* 3e */
	unsigned _prev_xtnt;		/* 40 */
	unsigned _ext_prev_xtnt;	/* 42 */
	unsigned _next_xtnt;		/* 44 */
	unsigned _ext_next_xtnt;	/* 46 */
	unsigned _size_xtnt;		/* 48 */
	unsigned _ext_size_xtnt;	/* 4a */
	unsigned _bound_xtnt;		/* 4c */
	unsigned _ext_bound_xtnt;	/* 4e */
	char _data[512];			/* 50 */
	};

struct file_desc {
	char _Drive;				/* 00 */
	char _Mode;					/* 01 */
	char _Owner;				/* 02 */
	char _Group;				/* 03 */
	char _Fspare[2];			/* 04 */
	unsigned _First_xtnt;		/* 06 */
	unsigned _ext_First_xtnt;	/* 08 */
	unsigned _Last_xtnt;		/* 0a */
	unsigned _ext_Last_xtnt;	/* 0c */
	unsigned _Num_blks;			/* 0e */
	unsigned _ext_Num_blks;		/* 10 */
	unsigned _Num_xtnt;			/* 12 */
	unsigned _Cur_xtnt;			/* 14 */
	unsigned _ext_Cur_xtnt;		/* 16 */
	unsigned _Max_cnt;			/* 18 */
	unsigned _Cur_blk_offset;	/* 1a */
	unsigned _Parent_xtnt;		/* 1c */
	unsigned _ext_Parent_xtnt;	/* 1e */
	unsigned _Xhdr_list;		/* 20 */
	unsigned _Fp_flags;			/* 22 */
	unsigned _Prev_num_xtnts;	/* 24 */
	unsigned _Prev_num_blks;	/* 26 */
	int _Dir_index;				/* 28 */
	int _Temp;					/* 2a */
	unsigned _Prev_xtnt;		/* 2c */
	unsigned _ext_Prev_xtnt;	/* 2e */
	unsigned _Next_xtnt;		/* 30 */
	unsigned _ext_Next_xtnt;	/* 32 */
	unsigned _Size_xtnt;		/* 34 */
	unsigned _ext_Size_xtnt;	/* 36 */
	unsigned _Bound_xtnt;		/* 38 */
	unsigned _ext_Bound_xtnt;	/* 3a */
	} ;

	/* total size 0x3c */

struct file_entry {
	struct file_desc fdesc;		/* 00 */
	char fe_flag_save;			/* 3c */
	char fflags;				/* 3d */
	unsigned user_tid;			/* 3e */
	} ;
	/* total size 0x40 */

#define FILE_USED	1
#define DIR_CHANGED	2

struct r_w_msg {
	char mtype;
	unsigned r_w_drive;
	unsigned r_w_block;
	unsigned r_w_ext_block;
	char r_w_filler[4];
	char r_w_data[0];
	};

struct adopt_entry {
	unsigned adopt_nid;
	unsigned adopt_tid;
	char adopt_drive;
	} ;

struct adopt_drive_msg {
	char mtype;
	char a_drive;
	struct adopt_entry a_entry;
	} ;

struct dcb {
	unsigned dcb_admin;
	unsigned dcb_nid;
	unsigned dcb_tid;
	char dcb_drive;
	char dcb_spare[5];
	} ;

struct cd_msg {
	char mtype;
	char cd_msg_filler[10];
	char cd_name[0];
	};

struct cd_entry {
	char cur_group;
	char cur_owner;
	char cur_drive;
	unsigned cur_parent_xtnt;
	unsigned cur_dir_index;
	unsigned cur_disk_id[2];
	} ;

struct disk_entry {
	char disk_type;			/* 01 */
	char disk_drv;			/* 02 */
	unsigned blk_offset;	/* 03 */
	unsigned num_sctrs;		/* 05 */
	unsigned sctr_cyl;		/* 07 */
	char sctr_trk;			/* 09 */
	char sctr_base;			/* 0a */
	unsigned ctl_addr;		/* 0b */
	char ctl_int;			/* 0d */
	unsigned num_tracks;	/* 0e */
	char disk_filler[1];	/* 0f */
	} ;

struct ldisk_entry {
	struct disk_entry _disk_entry;
	unsigned ext_blk_offset;
	unsigned ext_num_sctrs;
	char disk_ext_filler[10];
	unsigned disk_entry_id;
	} ;

struct disk_entry_ext {
	unsigned _ext_blk_offset;
	unsigned _ext_num_sctrs;
	char _disk_ext_filler[10];
	unsigned _disk_entry_id;
	} ;

struct driver_entry {
	unsigned driver_seg;		/* 00 */
	int (*disk_init) ();		/* 02 */
	int (*blk_read) ();			/* 04 */
	int (*blk_write) ();		/* 06 */
	int (*format_track) ();		/* 08 */
	unsigned driver_entry_spare[3];	/*0a */
	int (*driver_user_fun) ();	/* 10 */
	} ;
/* total size = 0x12 */

struct driver_msg {
	char mtype;
	char driver_type;
	struct driver_entry drv_ent;
	} ;

struct cache_ctrl_entry {
	char cache_flags;
	unsigned cache_disk_id;
	} ;

#define CACHE_ON		0x01
#define CACHE_REMOVABLE	0x02

struct mount_cache_msg {
	char mtype;
	unsigned cache_size;
	unsigned cache_segment;
	char cache_drive_number;
	} ;

struct purge_cache_msg {
	char mtype;
	char purge_drive_number;
	} ;

struct format_track_msg {
	char mtype;
	char form_drive;
	int  form_track;
	char form_head;
	char form_sectors;
	char form_stagger_buf[0]; /* stagger table, 0..n-1 */
	} ;

/*
 * define file status messages ("ored" with ERROR on return)
 */

#define UNABLE_TO_ACCESS	1
#define FILE_BUSY			2
#define NO_BUF_SPACE		3
#define BAD_PATHNAME		4
#define ATTRIBUTE_VIOLATION	5
#define DISK_FULL			6
#define NAME_TOO_LONG		7
#define INVALID_DEVICE		8
#define BAD_DRIVE_NUMBER	9
#define PARTIAL_MATCH		10
#define TOO_MANY_OPEN_FILES	11
#define CORRUPT_DIRECTORY	12

