#define FILE	struct iocb

#define EOF -1
#define NULL 0

/*
 * Device types
 */

#define TYPE_FILE	0
#define TYPE_DEVICE	1

#define _READ			0x01
#define _WRITE			0x02
#define _APPEND			0x04
#define _CREATE			0x04
#define _EXECUTE		0x08
#define _BLOCK			0x08
#define _MODIFY			0x10
#define _DIRECTORY		0x20

#define _NEW			0x01
#define _OLD			0x02
#define _FILE			0x04
#define _VERBOSE		0x08
#define _QUIT			0x10
#define _CACHE			0x20

#define _EOF			0x80

#define _UNBUF			0x40
#define _UNGOT			0x20
#define _DIRTY			0x10
#define _FILTHY			0x08
#define _EOF_PENDING	0x08
#define _SLIMY			0x04



/*
 * This is the basic IO control block. It is the minimum
 * required information for IO transferrs. The buffer is
 * of variable length depending on the particular device.
 */

struct iocb {
	struct iocb *_link;		/* 00 */
	char _unchar;			/* 02 */
	char _dev_type;			/* 03 */
	char _cmd_stat;			/* 04 */
	unsigned _arg[2];		/* 05 */
	char _flags;			/* 09 */
	char _dev_no;			/* 0a */
	int _index;				/* 0b */
	int _length;			/* 0d */
	char _drive_number;		/* 0f */
	char _iocb_spare;		/* 10 */
	unsigned _admin_tid;	/* 12 */	/* new field */
	char _buffer[0];		/* 14 */
	};	/* 14 + _buffer */

struct msg_hdr {
	char mtype;
	unsigned marg[2];
	char mflags;
	char mdev_no;
	int mindex;
	int mlen;
	char mdrive_number;
	char miocb_spare;
	unsigned mtid;
	char mbuf[0];
	} ;

#define FMSG_HDR sizeof(struct msg_hdr)


/*
 * Messages
 */

#define TASK_DIED			0
#define FILL				1
#define EMPTY				2
#define OPEN				3
#define CLOSE				4
#define SEEK				5
#define REWIND				6
#define FORWARD				7
#define GET_ATTR			8
#define SET_ATTR			9
#define CLOSE_ALL			10
#define ABORT_MSG			11
#define CHAR_WAITING		12
#define GET_OPTION			13
#define READ_BLK			13
#define SET_OPTION			14
#define WRITE_BLK			14
#define DISK_PROMPT			15
#define LOAD				15
#define NEW_OPTION			16
#define CD					16
#define NEW_WINDOW			17
#define PASS_CD				17
#define ADOPT_DEVICE		18
#define DEFINE_DRIVER		18
#define INHERIT_DEVICE		19
#define GET_SEARCH			19
#define UNADOPT_DEVICE		20
#define SET_SEARCH			20
#define QUERY_DEVICE		21
#define GET_DIR_ENTRY		21
#define SET_DIR_ENTRY		22
#define LSEEK				23
#define MOUNT_XHDR_CACHE	24
#define MOUNT_CACHE			25
#define PWD					26
#define GET_SEARCH_ORDER	27
#define FTELL				28
#define FORMAT_TRACK		29
#define FLUSH_INPUT			29
#define STTY_MESSAGE		30
#define PURGE_CACHE			30
#define ADOPT_DRIVE			31
#define GET_RSEARCH			32
#define SET_RSEARCH			33
#define LREAD_BLK			34
#define LWRITE_BLK			35
#define LGET_ATTR			36

/*
 * Replies
 */

#define ERROR				0x80
#define RETRY_OPEN			0x40
#define OK					0
#define RETRY				1
#define READ_ERROR			2
#define WRITE_ERROR			3
#define FULL_DISK			4
#define BAD_XTNT			5
#define FILE_WRITE_ERROR	6
#define BAD_FILE_NUMBER     7
#define BAD_IOCB			8
#define COMMAND_ABORTED		9
