/*
 * Messages to the TASK ADMINISTRATOR.
 * These MUST NOT be changed once the system has been released.
 */

#define TA_DESTROY				0
#define TA_GROW_MEM				1
#define TA_CREATE				2
#define TA_TRANSFORM			3
#define TA_FORK					4
#define TA_CODE_INFO			5
#define TA_TASK_INFO			6
#define TA_ALLOC_SEG			7
#define TA_ADD_NEW_MEM			8
#define TA_MEM_INFO				9
#define TA_PASSWORD_ON			10
#define TA_WAIT					11
#define TA_FREE_SEG				12
#define TA_GET_ACCOUNT			13
#define TA_SET_ACCOUNT			14
#define TA_GET_ACC_MSG			15
#define TA_CHANGE_DIR			16	/* may be forwarded from fsys */
#define TA_REGISTER				17
#define TA_DREGISTER			18
#define TA_SYS_FLAGS			19


/*
 * Reply messages from the TASK ADMINISTRATOR.
 * These MUST NOT be changed once the system has been released.
 */

#define OK				0
#define NO_TID			1
#define NO_MEMORY		2
#define BAD_LOAD_FILE	3
#define CMD_NOT_FOUND	4
#define NOT_A_LOAD_FILE	5
#define BAD_MSG_TYPE	6
#define NO_8087			7
#define NOT_AVAILABLE	8


/*
 * Flag definitions for the the cmd_flags field of a TA_CREATE msg.
 */

#define RUN_BACKGROUND	0x01
#define RUN_CONCURRENT	0x02
#define BEQUEATH_TTY	0x04
#define QUERY			0x08
#define DEBUG           0x10


/*
 * Misc constants
 */

#define CMD_LINE_SIZE	256


struct ta_create {
	char msg_type;			/* 00 */
	char cmd_flags;			/* 01 */
	char cmd_priority;		/* 02 */
	char argc;				/* 03 */
	char stdin_index;		/* 04 */
	char stdout_index;		/* 05 */
	char stderr_index;		/* 06 */
	unsigned stack_overide;	/* 07 */
	int line_length;		/* 09 */
	char cmd_tty;			/* 0b */		/* IF 0 THEN current tty, ELSE tty - 1 */
	char tac_spare[11];		/* 0c */
	char cmd_line[CMD_LINE_SIZE];	/* 17 */
	} ;

struct tar_create {
	char msg_type;
	unsigned task_id;
	unsigned task_status;
	} ;
/* 5 */

struct ta_task_info {
	char msg_type;
	char task_num;
	} ;

struct ta_code_info {
	char msg_type;
	char code_num;
	} ;

struct tar_code_info {
	char msg_type;
	struct code_entry code_data;
	char code_pathname[100];
	} ;

struct ta_grow_mem {
	char msg_type;
	unsigned req_nbytes;
	} ;

struct ta_alloc_seg {
	char msg_type;
	unsigned req_nblks;
	char req_flags;
	unsigned req_segment;
	} ;

struct tar_alloc_seg {
	char msg_type;
	unsigned seg_base;
	} ;

struct ta_add_new_mem {
	char msg_type;
	unsigned new_seg;
	unsigned num_blks;
	} ;

struct ta_account {
	char msg_type;
	char accteam;
	struct account_entry account_data;
	} ;

struct ta_account_msg {
	char msg_type;
	char accteam;
	unsigned accflags;
	char account_msg[80];
	} ;

struct ta_register {
	char msg_type;
	unsigned reg_nid;
	unsigned reg_tid;
	unsigned reg_dat;
	unsigned reg_name[9];
	} ;
/* 25 */

struct ta_vc_create {
	char msg_type;
	unsigned remote_node_id;
	unsigned remote_task_id;
	unsigned max_msg_size;
	} ;

struct ta_sys_flags {
	char msg_type;
	unsigned clr_sys_flags;
	unsigned set_sys_flags;
	} ;
