/*
 * Timer message types
 * All types but TIMER_WAKEUP will reply immediatly.
 * TIMER_WAKEUP			- Reply when the timer triggers
 * TIMER_FORCE_READY	- Force the task ready when the timer triggers. If
 *						  the task is already ready then ignore. See
 *						  the kernel routine FORCE_READY. When used with
 *						  CANCEL this can be used to perform timed
 *						  sends/receives.
 * TIMER_SET_EXCEPTION	- Set the indicated exception when the timer triggers.
 * TIMER_SIGNAL			- Signal the indicated port when the timer triggers.
 * TIMER_CANCEL			- Cancel the indicated timer (absolute or relative).
 */

#define TIMER_WAKEUP		0
#define TIMER_FORCE_READY	1
#define TIMER_SET_EXCEPTION	2
#define TIMER_SIGNAL		3
#define TIMER_CANCEL		4

/*
 * Timer times may be either absolute or realtive. Each task is allowed
 * one request of each type to be pending.
 *
 * An absolute request will triger on the date and time specified. The
 * time is specified in the standard form returned by GET_DATE. It has
 * a minimum resolution of 1 sec. 
 *
 * A relative request is specified in clock ticks (50 msec/tick) and is
 * independant of the date. The maximum wait is 2**16 - 1 ticks or just
 * over one hour.
 */

#define ABSOLUTE			0
#define RELATIVE			1


struct timer_msg {
	char msg_type;
	char time_type;
	union {
		unsigned time_date[2];
		unsigned time_ticks;
		} time_value;
	union {
		unsigned time_exception[2];
		unsigned time_port;
		} time_data;
	} ;
